/******************************************************************************
 $Header:: /MewAddOn/Multi Edit/dataflex.s 18    2-02-98 22:02 Gerard         $

 Copyright (c) 1997 G.J. Wijnands. All rights reserved.

														MULTI-EDIT MACRO FILE

 Name:   DataFlex

 Description:  This macro file contains language support macros for DataFlex.
               In particular the functions for the properties.

 EXTERNAL functions:

 dfSetProperties - A function which enables the user to enable or disable
                   certain functions
 dfSetX          - Sets up the DataFlex specific global variables
 DataFlexInstall - To start the installation of the DataFlex add-on package

 INTERNAL functions:

 Df_Dialog          - A function used for testing
 dfGetProperties    - Get the specific language properties for the current
                      file.
 df_CloseParen      - Highlights to matching open paren when closing paren is
                      entered.
 _dfGetMatchPat     - Called by LangDoMatch
 DelimitedString    - Replaces the delimiter ";" in a string for another
                      delimiter

******************************************************************************/
macro_file DataFlex;

// MEW
#include LANGUAGE.SH
#include DBTOOLS.SH
#include MEW.SH

// Add-on
#include DATAFLEX.SH
#include DFREG.SH
#include DFINTER.SH

prototype DataFlex {
  str ReplaceDelimiter( str DelString, str NewDelimiter, ... );
  void ProcessExtensions();
}

str GetVDFRootDir( str sVDFRootdirKey )
/********************************************************************
                                                       Multi-Edit Macro
                                                         24-1-98 18:46

  Function: This function returns the rootdirectory of VDF binaries.
  Entry   : Registry key which contains the first part of the directory.
  Exit    : rootdirectory

                                    Copyright 1998 By Gerard J Wijnands
***********************************************************************/
{
  str sResult;
    
  sResult = GetStringFromGlobalRegistry( sVDFRootdirKey );
  if ( Length( sResult ) != 0) {
    sResult += "\\BIN";
  }

  return ( sResult );

}  // GetVDFRootDir

/******************************************************************************
															 Multi-Edit Macro
                               29-Nov-96  21:13

  Function: Get the specific DataFlex properties for the current file.
	Entry   : struct rCP		- A structure to fill with the properties.
  Exit    : -

               Copyright (C) 1996 by G.J. Wijnands
******************************************************************************/
void dfGetProperties()
{
  str sOnlineHelpfileKey;
  str sOnlineHelpfilePath;
  str sVDFRootdirKey;
  str sRootRegKey;
  int iPos;

  if ( gsProject != gsSESSIONNAME ) {
  
    gbClosureHighlight  = GetIntegerFromRegistry( gsREGFLDPROPERTIES, gsREGCLOSUREHIGHLIGHT );
    gbConstructMatching = GetIntegerFromRegistry( gsREGFLDPROPERTIES, gsREGCONSTRUCTMATCHING );
    
    gbDebug             = GetIntegerFromRegistry( gsREGFLDPROPERTIES, gsREGDEBUG );
      
    // ---------------------------------------------------------------------
    // Temporarily set the variables here out of nowhere...
  
    // ---------------------------------------------------------------------
    // Indent
    // 
    gbAutoUnindent          = GetIntegerFromRegistry( gsREGFLDPROPERTIES, gsREGAUTOUNINDENT);
    gsIndentKeywords        = " " + 
                              Remove_Space( GetStringFromRegistry( gsREGFLDPROPERTIES, gsREGINDENTKEYWORDS ))
                              + " ";
    gsUnindentKeywords      = " " + 
                              Remove_Space( GetStringFromRegistry( gsREGFLDPROPERTIES, gsREGUNINDENTKEYWORDS ))
                              + " ";
    gsSpecialIndentKeywords = " " + 
                              Remove_Space( GetStringFromRegistry( gsREGFLDPROPERTIES, gsREGSPECIALINDENTKEYWORDS ))
                              + " ";
      
    // ---------------------------------------------------------------------
    // EOL comment
    // 
    gbAutoEOLComment      = GetIntegerFromRegistry( gsREGFLDPROPERTIES, gsREGAUTOEOLCOMMENT     );
    gbEOLCommentHeader    = GetIntegerFromRegistry( gsREGFLDPROPERTIES, gsREGEOLCOMMENTHEADER   );
    giEOLHeaderWidth      = GetIntegerFromRegistry( gsREGFLDPROPERTIES, gsREGEOLHEADERWIDTH     );
    gbEOLAutoFillComment  = GetIntegerFromRegistry( gsREGFLDPROPERTIES, gsREGEOLAUTOFILLCOMMENT );
    gsEOLAutoFillKeywords = " " + 
                            Remove_Space( GetStringFromRegistry( gsREGFLDPROPERTIES, gsREGEOLAUTOFILLKEYWORDS ))
                            + " ";
    gbEOLAdjustComment    = GetIntegerFromRegistry( gsREGFLDPROPERTIES, gsREGEOLADJUSTCOMMENT );
    giEOLAdjustColumn     = GetIntegerFromRegistry( gsREGFLDPROPERTIES, gsREGEOLADJUSTCOLUMN  );

    gsEOLAutoFillTags = "(" + ReplaceDelimiter(
                          ConvertToRegExp( GetStringFromRegistry( gsREGFLDPROPERTIES, gsREGEOLAUTOFILLTAGS ))
                          , ")|(", " " ) + ")";

    // ---------------------------------------------------------------------
    // Tags
    // 
    gsTagKeywords = "(" + ReplaceDelimiter( ConvertToRegExp(
                    GetStringFromRegistry( gsREGFLDPROPERTIES, gsREGEOLAUTOFILLTAGS ) +
                    GetStringFromRegistry( gsREGFLDPROPERTIES, gsREGTAGKEYWORDS ))
                          , ")|(", " " ) + ")";

    // ---------------------------------------------------------------------
    // ??
    // 

    gsProject       = gsSESSIONNAME;
    gsLanguage      = GetSessionValue( gsProject, gsREGLANGUAGE );
    gsInherit       = GetSessionValue( gsProject, gsREGLANGUAGE );
    
    sRootRegKey     = GetStringSettingRecursively( gsREGROOTREGKEY );
    gsWorkSpacesKey = sRootRegKey + GetStringSettingRecursively( gsREGWORKSPACESKEY );

    gsWorkspace     = GetStringSettingRecursively( gsREGWORKSPACE     );
    gsMainApp       = GetStringSettingRecursively( gsREGMAINAPP       );
    gsType          = GetStringSettingRecursively( gsREGTYPE          );
    if ( gsType == gsTYPEWINDOWS95) {
      gsDFPath       = GetStringFromGlobalRegistry( gsMainAppKey + "\\" + gsMainApp + "\\DFPath" );
      gsMakePath     = GetStringFromGlobalRegistry( gsMainAppKey + "\\" + gsMainApp + "\\MakePath" );
      gsDFProg       = "";
      gsDFEnv        = "";
      gsHome         = "";
      gsDDSearchPath = GetStringFromGlobalRegistry( gsWorkSpacesKey + "\\" + gsWorkspace + "\\DDSrcPath" );
      gsMainAppKey    = sRootRegKey + GetStringSettingRecursively( gsREGMAINAPPKEY    );
    
      gsMainAppSkip   = GetStringSettingRecursively( gsREGMAINAPPSKIP );
      if ( length( gsMainAppSkip ) != 0) {
        gsMainAppSkip = "," + gsMainAppSkip + ",";
      }
      if ( GetIntegerSettingRecursively( gsREGBUSEPROGRAMPATH )) {
        gsProgramPath = GetStringFromGlobalRegistry( gsWorkSpacesKey + "\\" + gsWorkspace + "\\ProgramPath" );
        if ( iPos = XPos( ";", gsProgramPath, 1 )) {
          gsProgramPath = Copy( gsProgramPath, 1, iPos - 1 );
        }
        if ( Copy( gsProgramPath, Length(gsProgramPath), 1 ) != "\\" ) {
          gsProgramPath += "\\";
        }
      }
      else gsProgramPath = "";
    }
    else {
      gsDFPath       = GetStringSettingRecursively( gsREGDFPATH        );
      gsMakePath     = GetStringSettingRecursively( gsREGMAKEPATH      );
      gsDFProg       = GetStringSettingRecursively( gsREGDFPROG        );
      gsDFEnv        = GetStringSettingRecursively( gsREGDFENV         );
      gsHome         = GetStringSettingRecursively( gsREGHOME          );
      gsDDSearchPath = "";
      gsProgramPath  = "";
      gsMainAppSkip  = "";
    }
    
    gsSearchPath    = GetStringSettingRecursively( gsREGSEARCHPATH    );
    gsDDSearchPath  = GetStringSettingRecursively( gsREGDDSEARCHPATH  );
    gsDFComp        = GetStringSettingRecursively( gsREGDFCOMP        );
    gsDFPreComp     = GetStringSettingRecursively( gsREGDFPRECOMP     );
    gsBugDFComp     = GetStringSettingRecursively( gsREGBUGDFCOMP     );
    gsExplorer      = GetStringSettingRecursively( gsREGEXPLORER      );
    gsDFIndex       = GetStringSettingRecursively( gsREGDFINDEX       );
    gsDFFile        = GetStringSettingRecursively( gsREGDFFILE        );
    gsDFMaint       = GetStringSettingRecursively( gsREGDFMAINT       );
    gsAppBuilder    = GetStringSettingRecursively( gsREGAPPBUILDER    );

    gsDFSpy         = GetStringSettingRecursively( gsREGDFSPY         );
    gsCompiler      = GetStringSettingRecursively( gsREGCOMPILER      );
    gsBugCompiler   = GetStringSettingRecursively( gsREGBUGCOMPILER   );
    gsRuntime       = GetStringSettingRecursively( gsREGRUNTIME       );
    gsRuntimeParams = GetStringSettingRecursively( gsREGRUNTIMEPARAMS );
    
    // -------------------------------------------------------------------
    // Determine onlinehelp file
    // 
    sOnlineHelpfileKey  = sRootRegKey + GetStringSettingRecursively( gsREGONLINEHELPFILEKEY );
    sOnlineHelpfilePath = GetStringFromGlobalRegistry( sOnlineHelpfileKey );
    gsOnlineHelpfile    = GetStringSettingRecursively( gsREGONLINEHELPFILE );
    if ( Length( sOnlineHelpfilePath ) != 0 ){
      gsOnlineHelpfile = sOnlineHelpfilePath + "\\" + gsOnlineHelpfile;
    }
 
    // -------------------------------------------------------------------
    // Determine VDFRoot dir
    // 
    sVDFRootdirKey  = sRootRegKey + GetStringSettingRecursively( gsREGVDFROOTDIRKEY );
    gsVDFRootDir    = GetVDFRootDir( sVDFRootdirKey );

    gsSearchstring  = GetStringSettingRecursively( gsREGSEARCHSTRING  );
    gbDFEnv         = GetIntegerSettingRecursively( gsREGBDFENV     );

    if ( Length( gsSearchPath ) == 0) {
      gsSearchPath = gsMakePath;
    }
    if ( Length( gsDDSearchPath ) == 0) {
      gsDDSearchPath = gsSearchPath;
    }
  }
}  // dfGetProperties

void RefreshSettings( )
/********************************************************************
                                                       Multi-Edit Macro
                                                       13-01-1998 19:56

  Function: This function will refresh all settings in the global variables
            in MEW from the registry.
  Entry   : -
  Exit    : -


***********************************************************************/
{
  gsProject = "";
  dfGetProperties;

}  // RefreshSettings

/******************************************************************************
															 Multi-Edit Macro
                               17-Dec-96  21:09

  Function: Replaces the delimiter ";" in a string for another delimiter

  Entry   : str DelString    - The delimited string
            str NewDelimiter - The new delimiter
            str OldDelimiter - The old delimiter (optional)

  Exit    : str   - The new string

               Copyright (C) 1996 by G.J. Wijnands
******************************************************************************/
str ReplaceDelimiter( str DelString, str NewDelimiter, ... )
{
  int DelPos;                           // The position of the old delimiter
  str OldDelimiter;                     // The old delimiter
  int OldLength;                        // Length of the old delimiter

  if (Get_Param_Type( 1 ) == 1) {
    OldDelimiter = Get_Param_Str( 1 );
  }  
  else {
    OldDelimiter = ";";
  }
  OldLength = svl( OldDelimiter );

  // ---------------------------------------------------------------------
  // Remove a first delimiter
  // 
  if ( Copy( DelString, 1, OldLength ) == OldDelimiter )  {
    DelString = Str_Del( DelString, 1, OldLength );
  }

  // ---------------------------------------------------------------------
  // Remove a last delimiter
  // 
  if ( Str_Char( DelString, svl( DelString) - OldLength ) == OldDelimiter )  {
    DelString = Str_Del( DelString, svl( DelString) - OldLength, OldLength );
  }

  // ---------------------------------------------------------------------
  // Replace all double old delimiters with one
  // 
  while ( DelPos = XPos( OldDelimiter + OldDelimiter, DelString, 1 ))
  {
    DelString = Str_Del( DelString, DelPos, OldLength );
  }

  // ---------------------------------------------------------------------
  // Replace all old delimiters with the new one.
  // 
  while ( DelPos = XPos( OldDelimiter, DelString, 1 ))
  {
    DelString = Str_Del( DelString, DelPos, OldLength);      // Remove the old delimiter
    DelString = Str_Ins( NewDelimiter, DelString, DelPos );  // Replace it with the new one
  }
  return ( DelString );
}  // ReplaceDelimiter

/******************************************************************************
															 Multi-Edit Macro
															 05-Apr-96  11:23

  Function: Sets the DataFlex language construct matching globals.
  Entry   : None
  Exit    : None

               Copyright (C) 1996 by G.J. Wijnands
******************************************************************************/
void dfSetX( )
{
  str KeywordsBegPat[Max_Line_Length],KeywordsEndPat[Max_Line_Length];
  str KeywordFirst,KeywordLast;
  int Number,Position;

  dfGetProperties;

  if ( gbClosureHighlight )
  {
    Key_To_Window( Ascii( ")" ), "DATAFLEX^df_CloseParen /LP="+gsLANGUAGESHORT+" /BR=)" );  // ")" key
    Key_To_Window( Ascii( "}" ), "DATAFLEX^df_CloseParen /LP="+gsLANGUAGESHORT+" /BR=}" );  // "}" key
    Key_To_Window( Ascii( "]" ), "DATAFLEX^df_CloseParen /LP="+gsLANGUAGESHORT+" /BR=]" );  // "]" key
  }

  KeywordsBegPat = "";
  KeywordsEndPat = "";

  if ( Length( Global_Str( "!dfMatchExtra" ) ) == 0 ) {
    if ( gbConstructMatching )
    {
      Number=0;
      while ( Length( KeywordFirst = GetStringFromRegistry( gsREGFLDKEYWORDS, Str(++Number) ) ) != 0 )
      {
        KeywordFirst = caps(KeywordFirst);
        Position = XPOS(",",KeywordFirst,1);
        if ( Position )
        {
          
          KeywordLast  = Copy(KeywordFirst,Position+1,svl(KeywordFirst)-Position);
          KeywordFirst = Copy(KeywordFirst,1,Position-1);
          KeywordsBegPat = KeywordsBegPat +
              _db_Delimit + ReplaceDelimiter(KeywordFirst,_db_Delimit) +
                  _mtchBeg + " " + ReplaceDelimiter(KeywordFirst," ") + " " +
                  _mtchMid +
                  _mtchEnd + " " + ReplaceDelimiter(KeywordLast," ") + " " +
                  _mtchExp + "(" + ConvertToRegExp( ReplaceDelimiter(KeywordFirst,")|(") + ")|(" + ReplaceDelimiter(KeywordLast,")|(")) + ")";
          KeywordsEndPat = KeywordsEndPat +
              _db_Delimit + ReplaceDelimiter(KeywordLast,_db_Delimit) +
                  _mtchBeg + " " + ReplaceDelimiter(KeywordLast," ") + " " +
                  _mtchEnd + " " + ReplaceDelimiter(KeywordFirst," ") + " " +
                  _mtchExp + "(" + ConvertToRegExp( ReplaceDelimiter(KeywordFirst,")|(") + ")|(" + ReplaceDelimiter(KeywordLast,")|(")) + ")";
        }
      }
    }

    // Set word begin/end characters
    Set_Global_Str( "!dfMatchExtra",
        _mtchBChars + gsDFEndOfWord +
        _mtchEChars + gsDFEndOfWord );
		// Set begin construct patterns
    Set_Global_Str( "!dfMatchBegPat",
        _db_Delimit + "(" +                          // ( matching
            _mtchBeg + " ( " +
            _mtchMid +
            _mtchEnd + " ) " +
            _mtchExp + "[()]" +
        _db_Delimit + "[" +                          // [ matching
            _mtchBeg + " [ " +
            _mtchMid +
            _mtchEnd + " ] " +
            _mtchExp + "[\\[\\]]" +
        _db_Delimit + "{" +                         // { matching
            _mtchBeg + " { " +
            _mtchMid +
            _mtchEnd + " } " +
            _mtchExp + "[{}]" +
        KeywordsBegPat +
        _db_Delimit );
		// Set end construct patterns
    Set_Global_Str( "!dfMatchEndPat",
        _db_Delimit + ")" +                          // ) matching
            _mtchBeg + " ) " +
            _mtchEnd + " ( " +
            _mtchExp + "[()]" +
        _db_Delimit + "]" +                          // ] matching
            _mtchBeg + " ] " +
            _mtchEnd + " [ " +
            _mtchExp + "[\\[\\]]" +
        _db_Delimit + "}" +                          // } matching
            _mtchBeg + " } " +
            _mtchEnd + " { " +
            _mtchExp + "[{}]" +
        KeywordsEndPat +
        _db_Delimit );
	}
}  // dfSetX 

/******************************************************************************
															 Multi-Edit Macro
                               30-Nov-96  20:45

	Function: Auto match the opening paren for the specified language.

	Entry   : str LangPrefix	- The first three letters of the language (/LP=)
            str Brace       - The type of brace used. (/BR=)
						str ParmStr			- Additional parameters ( See LangDoMatch )

	Exit    : None

               Copyright (C) 1996 by G.J. Wijnands
******************************************************************************/
void df_CloseParen(
			str LangPrefix = Parse_Str( "/LP=", MParm_Str ),
      str Brace      = Parse_Str( "/BR=", MParm_Str ),
			str ParmStr 	 = MParm_Str
)
{
	int SavRefresh = Refresh;
	int TSearch_Highlight;

  Push_Undo;
	Refresh = False;
  Text( Brace );
  if ( ( ( GetStatusAtCursor( ) & 0x0F ) == 0 ) && Svl( LangPrefix ) ) {
      Left;
      LangDoMatch( gsLANGUAGESHORT, Brace, ParmStr + "/RC=1/HI=1/LS=20" );
			TSearch_Highlight = Search_Highlight;
			Right;
			Search_Highlight = TSearch_Highlight;
  }
  Refresh = SavRefresh;
  Pop_Undo;
}  // df_CloseParen 

/******************************************************************************
															 Multi-Edit Macro
                               01-Dec-96  09:31

  Function: Called by LangDoMatch to locate and return a special pattern to
            match for the DataFlex language.

  Entry   : None

  Exit    : Return_Str  - Pattern to match or "" if a special pattern not found
            g_LangInCmt - True when pattern starts in a comment

               Copyright (C) 1996 by G.J. Wijnands
******************************************************************************/
void _dfGetMatchPat()
{
  str sSpecial = "{}()[]";
  str sTmp;

  // ---------------------------------------------------------------------
  // If special character then return it, otherwise go to the beginning of
  // the current word and return "".
  // 
  if ( XPos( Cur_Char, sSpecial,  1 ) ) {
    sTmp = _db_Delimit + Cur_Char + _db_Delimit;
  }
  else {
    if ( (At_EOL) and (C_Col > 1)) {
      Left;
    }
    // Move back to start of word
    while ( !XPos( Cur_Char, gsDFEndOfWord, 1 ) && ( C_Col != 1 ) ) {
      Left;
    }
    sTmp = "";
  }
  Return_Str = sTmp;
}  // _dfGetMatchPat

/******************************************************************************
															 Multi-Edit Macro
                               01-Dec-96  09:31

  Function: To start the installation of the DataFlex add-on package

  Entry   : None

  Exit    : None

               Copyright (C) 1997 by G.J. Wijnands
******************************************************************************/
void DataFlexInstall( ) trans2
{
  ProcessExtensions();
  
  Rm( "UPDATE^Install /A=1/F=" + Me_Path + "DATAFLEX\\INSTALL.LST" );

  Return_Str = me_path + "DATAFLEX\\DATAFLEX.TXT";
	Rm( "LdFiles /LC=0/CW=2/NRL=1" );
}  // DataFlexInstall

/******************************************************************************
															 Multi-Edit Macro
                               28-Sep-97  22:00

  Function: Enable / Disable menu-entries

  Entry   : None

  Exit    : True  - disable menu
            False - enable menu

               Copyright (C) 1997 by G.J. Wijnands
******************************************************************************/
int DisableCommand( str sMetaCommand )
{
  str sCommand;

  if (( gsLanguage == "" ) || ( gsProject  == "" )) sCommand = "";
  else sCommand = TranslateMetaCommands( sMetaCommand );

	return ( svl(sCommand)==0 );
}  // DisableCommand 

int DisableDataFlex()                   // The default
{
  return ( ( gsLanguage == "" ) ||
           ( gsProject  == "" ) );
}  // DisableDataFlex


int DataFlexExtension( str sExtension )
/********************************************************************
                                                       Multi-Edit Macro
                                                       15-01-1998 21:49

  Function: This function will check whether the language of the files
            with the supplied extension is DataFlex.
  Entry   : sExtension - the extension (without ".") to check
  Exit    : True  - Extension has DataFlex as lagnuage
            False - Extension has another language

***********************************************************************/
{
  return (Caps( Parse_Str( _db_Delimit + "LS=", Global_Str( "." + sExtension )) ) == gsLANGUAGELONG );
}  // DataFlexExtension 

int DisableRunprogram()
{
  return ( DisableDataFlex() || 
           !(DataFlexExtension( Get_Extension( File_Name ) )) ||
           ( gsProject == "" ) );
}

int DisableMenu()
{
  return ( (gsType == gsTYPEWINDOWS95) or (gsLanguage=="") or (gsProject == ""));

}  // DisableMenu

int DisableExplorer()
{
  return ( DisableCommand( "<EXPLORER>" ) );
}

int DisableDFIndex()
{
  return ( DisableCommand( "<DFINDEX>" ) );
}  // DisableDFIndex

int DisableDFFile()
{
  return ( DisableCommand( "<DFFILE>" ) );
}  // DisableDFFile

int DisableDFMaint()
{
  return ( DisableCommand( "<DFMAINT>" ) );
}  // DisableDFMaint 

int DisableAppBuilder()
{
  return ( DisableCommand( "<APPBUILDER>" ) );
}  // DisableDFMaint 

int DisableConfigureSession()
{
  return (DisableDataFlex());
}  // DisableConfigureSession 


/********************************************************************
                                                       Multi-Edit Macro
                                                       11-01-1998 19:10

  Function: This function does some conversions to a string to make it
            correct for the regular expressions. (# -> \#)
  Entry   : The string to convert
  Exit    : The converted string


***********************************************************************/
str ConvertToRegExp( str sSearch )
{
  str sTmp[ Max_Line_Length ];
  int iPos;

  sTmp = sSearch;
  iPos = 1;

  // ---------------------------------------------------------------------
  // Replace the '#' with '\#' to make it an exact match with regular
  // expressions.
  // 
  while ( iPos = XPos( "#", sTmp, iPos )) {
    sTmp = Str_Ins( '\', sTmp, iPos );
    iPos += 2;
  }

  return ( sTmp );
}  // ConvertToRegExp

void ForEachDataFlexExtension( int iNumRec, str sDo, int iConfigWin, int iDoWin )
/********************************************************************
                                                       Multi-Edit Macro
                                                         25-1-98 17:08

  Function: This function walks through the filename extensions and for
            each extension with the language DATAFLEX it runs the
            specified macro <sDo>.
  Entry   : `entry`
  Exit    : `exit`

                                    Copyright 1998 By Gerard J Wijnands
***********************************************************************/
{
  int iCount;

  Switch_Window( iConfigWin );
  mark_pos;
  for ( iCount=0; iCount < iNumRec; iCount++ ) {
    down;
    if ( Caps( Parse_Str( _db_Delimit + "LS=", Get_Line() )) == "DATAFLEX" ) {
      
      RM( "DataFlex^" + sDo + " /UW=" + str( iDoWin ) );
      
      Switch_Window( iConfigWin );
    }
  }
  goto_mark;

}  // ForEachDataFlexExtension

void UpdateExtension( int iUpdWin = Parse_Int( "/UW=", MParm_Str ) )
/********************************************************************
                                                       Multi-Edit Macro
                                                         25-1-98 17:11

  Function: This function writes an entry in the specified window which
            will make sure that for the filename extension (which is
            specified on the current line of the current window) the
            correct settings are installed.
  Entry   : iUpdWin - the windownumber of the ".UPD" file
  Exit    : -

                                    Copyright 1998 By Gerard J Wijnands
***********************************************************************/
{
  str sExt;

  sExt = Parse_Str( _db_Delimit + "EXT=", Get_Line() );
  
  Switch_Window( iUpdWin );
      
  Text( "Set Replace Dat EXT=" + sExt ); Cr;
  Text( "Set Fields I IND= ATX= HLP= ATEMP= DIR=" ); Cr;
  Text( "Dat EXT=" + sExt + "IND=3ATX=1HLP=<~!DFSETONLINEHELPFILE>ATEMP=DFADDONDIR=" );
  if ( sExt == "DD" ) {
    Text( "<~!DFSETDDSEARCHPATH>" );
  }
  else {
    Text( "<~!DFSETSEARCHPATH>" );
  }
  Cr;

}  // UpdateExtension

void UpdateCompiler( int iUpdWin = Parse_Int( "/UW=", MParm_Str ) )
/********************************************************************
                                                       Multi-Edit Macro
                                                         25-1-98 17:14

  Function: This function writes an entry in the specified window which
            will make sure that for the filename extension (which is
            specified on the current line of the current window) a
            compiler is installed.
  Entry   : iUpdWin - the windownumber of the ".UPD" file
  Exit    : -

                                    Copyright 1998 By Gerard J Wijnands
***********************************************************************/
{
  str sExt;

  sExt = Parse_Str( _db_Delimit + "EXT=", Get_Line() );
  
  Switch_Window( iUpdWin );
      
  Text( "DelPage " + sExt + ".PGM" );           Cr;
  Text( "Set Page " + sExt + ".PGM" );          Cr;
  Text( "Set Create True" );                    Cr;
  Text( "Set Update Dat PN=Compile" ); Cr;
  Text( "Dat PN=CompileCL=<~!DFCOMPILE> <~!DFCOMP>WORKLOC=1PT=DataFlexEXEID=1SHOW=1SA=1BKG=1" );
  Cr;
  Text( "Set Update Dat PN=Compile for debugging" ); Cr;
  Text( "Dat PN=Compile for debuggingCL=<~!DFCOMPILE> <~!DFCOMP> <~"+gsDUMDFDEBUG+">WORKLOC=1PT=DataFlexEXEID=1SHOW=1SA=1BKG=1" );
  Cr;
  Text( "Set Update Dat PN=Compile (ask DFComp)" ); Cr;
  Text( "Dat PN=Compile (ask DFComp)CL=<~!DFCOMPILE> <~!DFCOMP> <~"+gsASKDFCOMP+">WORKLOC=1PT=DataFlexEXEID=1SHOW=1SA=1BKG=1" );
  Cr;
  Text( "Set Update Dat PN=Compile - src" ); Cr;
  Text( "Dat PN=Compile - srcCL=<~!DFCOMPILER> <FILE>.<EXT> <~!DFCOMP>WORKLOC=1PT=DataFlexEXEID=1SHOW=1SA=1BKG=1" );
  Cr;
  Text( "Set Update Dat PN=Precompile" ); Cr;
  Text( "Dat PN=PrecompileCL=<~!DFCOMPILE> <FILE>.<EXT> <~!DFCOMP> <~"+gsDUMDFPRECOMP+">WORKLOC=1PT=DataFlexEXEID=1SHOW=1SA=1BKG=1" );
  Cr;

}  // UpdateCompiler

/********************************************************************
                                                       Multi-Edit Macro
                                                         24-1-98 23:25

  Function: This function will walk through the filename extensions
            and for each extension with language DATAFLEX an entry
            in DATAFLEX.UPD is created.
  Entry   : -
  Exit    : -

                                              Copyright 1998 By Gerard J Wijnands
***********************************************************************/
void ProcessExtensions()
{
	int OrgWin   = Window_Id;
  int iFirstRec = 0;
  int iNumRec;
	int TRefresh = Refresh;
  int iUpdWin, iConfigWin;
  int iOldBackups = Backups;

  Refresh = False;
  Backups = 0;
      
  Return_Str = ME_PATH + "\\DATAFLEX\\DATAFLEX.UPD";
	Rm( "LdFiles /LC=0/CW=2/NRL=1" );

  Tof;
  
  if (Find_Text( ";**INSERT HERE**", 0, 0 )) {
    iUpdWin = Window_Id;

    Del_Line;

    if ( DbGetPage( _ConfigFile, "EXT.DB", iFirstRec, iNumRec) == _NoError) {
      iConfigWin = Window_ID;
    
      Switch_Window( iUpdWin );
      Text( ";" );                                    Cr;
      Text( "; Automated Setup extensions " + Date ); Cr;
      Text( ";" );                                    Cr;
      Text( "Set Page EXT.DB" );                      Cr;

      ForEachDataFlexExtension( iNumRec, "UpdateExtension", iConfigWin, iUpdWin );
    
      Switch_Window( iUpdWin );
      Cr;
      Text( ";" );                                           Cr;
      Text( "; Automated compiler/program setup " + Date );  Cr;
      Text( ";" );                                           Cr;
    
      ForEachDataFlexExtension( iNumRec, "UpdateCompiler", iConfigWin, iUpdWin );
    }

    Switch_Window( iUpdWin );
    Save_File;
    if ( Error_Level != 0 ) RM('MEERROR');
  }  

  Delete_Window;

  Refresh = TRefresh;
  Backups = iOldBackups;
  Switch_Win_Id( OrgWin );

}  // ProcessExtensions


/********************************************************************
                                                     Multi-Edit Macro
                                                         3-5-98 17:22

  Function: Initializes our DataFlex Add-On
  Entry   : -
  Exit    : Global string '@Startup_Mac_2' gets set

                                    Copyright 1998 By Gerard J Wijnands
***********************************************************************/
void Init()
{
  Create_Global_Str( "@Startup_Mac_2", "DataFlex^RefreshSettings" );
} // Init 