/******************************************************************************
 $Header:: /MewAddOn/Multi Edit/dfind.s 2     25-01-98 16:55 Gerard           $

 Copyright (c) 1997 G.J. Wijnands. All rights reserved.

														MULTI-EDIT MACRO FILE

 Name:   `name`

 Description:  `description`

******************************************************************************/
macro_file DFIND;

// MEW
#include Language.sh

// Add-on
#include DataFlex.sh

prototype DFIND
{
  str  IndentKeyword();
  int  SkipNoiseFunc();
  int  ProcessEOLComment();
  void AutoUnindent( str C_Str );
}


int WordFound( str sWord, str sArray )
/********************************************************************
                                                       Multi-Edit Macro
                                                       13-01-1998 20:36

  Function: This function checks whether the word was found in the array.
            It's case insensetive.
  Entry   : sWord  - The word to be found
            sArray - A string of words seperated by spaces.
  Exit    : True  - The sArray contains the word
            False - The sArray does not contain the word.

***********************************************************************/
{
  return (XPos( " " + Caps( sWord ) + " ", sArray, 1 ) != 0);
}  // WordFound

void DFIndent( ) trans2
/******************************************************************************
															 Multi-Edit Macro
															 06-Dec-96  23:01

	Function: This macro will perform a smart indent when the <ENTER> key is
						pressed.  This macro is called by the macro CR.

	Entry   : `entry`
	Exit    : `exit`
	Author  : Dan Hughes

						Copyright (C) 1991-1996 by American Cybernectics, Inc.
 *******************************************************************( ldh )***/
{
  int  SavRefresh    = Refresh,
       SigCharFound,
       doAutoEolCmt,
       mark_count = 0,
       macro_without_ind = False,
       start_col,
       start_line;
  str  C_Str;          /* Word to check for indent */

  // ---------------------------------------------------------------------
  // Initialisation
  // 
  doAutoEolCmt = gbAutoEOLComment;
  messages     = FALSE;
  refresh      = FALSE;
  {mark_pos; ++mark_count;}

  // ---------------------------------------------------------------------
  // Not inserting a line, go to the first word on the next line
  // 
  if ( Insert_Mode == false ) {
    Cr;
    First_Word;
    goto Mac_Exit;
	}
  
  // ---------------------------------------------------------------------
  // If we are before or at the first character of the first word of the line
  // then leave the level the same for the line
  // 
  start_col = c_col;
  start_line = c_line;
  first_word;
  if ( ( c_col >= start_col ) && !at_eol ) {
    set_indent_level;
    cr;
    goto mac_exit;
  }

  // ---------------------------------------------------------------------
  // If we are at a directive command (#include or #replace), get out, but
  // not when we're at #$
  // 
  {mark_pos; ++mark_count;}
  first_word;
  if ( cur_char == '#' ) {
    right;
    if ( cur_char != '$' ) {
      {pop_mark; --mark_count;}
      set_indent_level;
      goto_col( start_col );
      cr;
      goto mac_exit;
    }
    else {
      macro_without_ind = True;
    }
  }
  {goto_mark; --mark_count;}

  goto_col( start_col );

  // ---------------------------------------------------------------------
  // Proces -optionally- the EOL comment
  //
  if ( doAutoEolCmt && (( GetStatusAtCursor( ) & _GSAC_EolComments ) != 0 )) {
    if (ProcessEOLComment()) goto mac_exit;
  }
  
  // ---------------------------------------------------------------------
  // Search for first word we can use.
  // 
  SigCharFound = SkipNoiseFunc();

  // ---------------------------------------------------------------------
  // If no word was found, set indent level to column 1
  // 
  if ( !SigCharFound ) {
    Goto_Col( 1 );
    Set_Indent_Level;
    {goto_mark; --mark_count; }
    Cr;
    goto Mac_Exit;
	}

  // ---------------------------------------------------------------------
  // Set the initial indentation
  // 
  First_Word;                           // Go to the first word on the line
  Set_Indent_Level;                     // Set the indent level

  // ---------------------------------------------------------------------
  // Go to the place where ENTER was hit.
  // 
  {goto_mark; --mark_count; }

  // ---------------------------------------------------------------------
  // Get the indentation keyword
  // 
  C_Str = IndentKeyword();

  // ---------------------------------------------------------------------
  // Perform Carriage-Return
  // 
  Cr;
  
  // ---------------------------------------------------------------------
  // Set the correct indenting level.
  // 
  if ( Svl( C_Str ) ) {
		// If the word is in this list then indent
    if ( WordFound( C_Str, gsIndentKeywords ) ) {
      Indent;
		}
    else if ( gbAutoUnindent ) {
      // Auto adjust end and until statements
      if ( WordFound( C_Str, gsUnindentKeywords ) ) AutoUnindent( C_Str );
    }
	}

// -----------------------------------------------------------------------
// Make sure that the marker-stack is left correctly and restore all
// other globals we don't want to mess with...
// 
Mac_Exit:
  while ( mark_count > 0 ) {
    pop_mark;
    --mark_count;
  }
  messages = TRUE;
  Refresh = SavRefresh;
	return ( );

}  // DFIndent


int SkipNoiseFunc()
/********************************************************************
                                                       Multi-Edit Macro
                                                       26-12-1997 17:16

  Function: This function will go back to the first word that is not
            in a comment.
  Entry   : -
  Exit    : True  - a word has been found
            False - nothing has been found

***********************************************************************/
{
  int    SigCharFound = false,
         InEolCmt,
         Jx;
  str    TStr[ Max_Line_Length ];
  struct TCommentChars RCC;
  
  GetCommentChars( RCC );               // Get comment characters

  while ( Find_Text( "[^" + gsDFEndOfWord + "]", 0, _regExp | _backward ) ) {
    if ( ( C_Line == 1 ) && ( c_col == 1 ) ) {
			// At top of file so get out
			break;
		}
    if ( Jx = ( GetStatusAtCursor( ) & _gsac_Comments ) ) {
			// In a comment
			switch ( Jx ) {
        case _gsac_BlkCmt1 :
          TStr = RCC.szCloseComment1;
          InEolCmt = false;
					break;

        case _gsac_BlkCmt2 :
          TStr = RCC.szCloseComment2;
          InEolCmt = false;
					break;

        case _gsac_EolCmt1 :
          TStr = RCC.szEolComment1;
          InEolCmt = true;
					break;

        case _gsac_EolCmt2 :
          TStr = RCC.szEolComment2;
          InEolCmt = true;
					break;

        case _gsac_EolCmt3 :
          TStr = RCC.szEolComment3;
          InEolCmt = true;
					break;

        case _gsac_EolCmt4 :
          TStr = RCC.szEolComment4;
          InEolCmt = true;
					break;
			}
      if ( InEolCmt ) {
				// Find the Eol comment characters
        Jx = Find_Text( TStr, 1, _backward );
			}
			else {
				// Find the opening block comment chars
        Jx = ( LangDoMatch( gsLANGUAGESHORT, TStr, "/NM=1/IC=0" ) == 0 );
			}
      if ( Jx ) {
        Left;
				continue;
			}
		}
    SigCharFound = true;
		break;
	}
  
  return ( SigCharFound );
}  // SkipNoiseFunc   

int ProcessEOLComment()
/********************************************************************
                                                       Multi-Edit Macro
                                                       26-12-1997 13:58

  Function: This function processes the making of the EOL-comment
  Entry   : -
  Exit    : True  - we are ready
            False - process as if there was no comment.

  The last position on the marker-stack must be the position where
  the ENTER-key was pressed.
  On exiting the marker-stack contains the same number of positions as
  on entering this function.

***********************************************************************/
{
  int delAutoCmt = FALSE;

  // ---------------------------------------------------------------------
  // Search the start of the EOL comment
  goto_col( 1 );
  while ( find_text( gsDFEOLComment, 1, 0 ) ) {
    if ( GetStatusAtCursor( ) & _GSAC_String ) {  // In quotes, skip
      right; right;
      continue;
    }
    break;
  }

  // Remember cursor position for later
  mark_pos;
  // -------------------------------------------------------------------
  // Move three spaces right.
  //
  right;
  right;
  right;

  if ( !at_eol )  { // we want to insert the slashes below the last
    goto_mark;                   // Back to the 1st slash
    set_indent_level;            // This is the new indent level
    goto_mark;                   // Back to where enter was pressed
    goto mac_exit;
  }
  // -------------------------------------------------------------------
  // MOVE ONE TO THE LEFT, and see if we are still at EOL.
  //
  left;
  if ( at_eol ) {
    // -----------------------------------------------------------------
    // OK. We are at the end of a line which has nothing except the double
    // slashes (i.e. not even a space after them).  This means that the
    // user typed //<CR>.  We will insert a dashed-line separator on the
    // current line, insert a <CR>, and put new double-slashes on the
    // next line.
    //
    goto_mark;
    set_indent_level;
    goto_mark;
    text( ' ' );
    if (gbEOLCommentHeader) {
      while ( c_col < giEOLHeaderWidth ) {
        text( '-' );
      }
    }
    goto mac_exit;
  }
  // -------------------------------------------------------------------
  // We got a CR on a line with only slashes and one space.
  // This means that the user wants to delete them IF they were inserted
  // automatically.  If so, we should act as if CR was pressed on the
  // line above (i.e. delete this line entirely).
  // We will do all that, and also set the indent level
    
  // -------------------------------------------------------------------
  // Determine if these were inserted automatically, by checking for slashes
  // immediately above these.  If there are, assume automatic slashes
  //
  goto_mark;                          // Get back to the 1st slash
  mark_pos;
  if ( c_line != 1 ) {
    up;
    if ( cur_char == '/' ) {
      right;
      if ( cur_char == '/' ) {
        delAutoCmt = TRUE;            // Flag says 'we did it'
        down;
        del_line;
        up;
        eol;
      }
    }
  }
  // -------------------------------------------------------------------
  // If we didn't delete some auto-slashes, then insert some!
  //
  if ( !delAutoCmt ) {
    goto_mark;                   // Back to the 1st slash
    set_indent_level;            // This is the new indent level
    goto_mark;                   // Back to where enter was pressed
    goto mac_exit;
  }
  // -------------------------------------------------------------------
  // We deleted a line which only had automatically inserted slashes, and
  // the cursor is sitting at EOL of the line above it.  Perform indenting
  // as if we never saw the EOL comment!
  //
  pop_mark;                     // Ignore the position of the 1st slash
  pop_mark;                     // pop the marker set on entry
  mark_pos;                     // replace it with the current position

  return( FALSE );

Mac_Exit:
  // ---------------------------------------------------------------------
  // Perform a carriage return and insert the comments for the next line.
  cr;
  text( gsDFEOLComment + " " );
  // ---------------------------------------------------------------------
  // Because one marker extra is popped from the stack, we push one on it
  // so there's a balance between the incomming stack and the outgoing.
  mark_pos;
  return( TRUE );
}  // ProcessEOLComment       


str IndentKeyword()
/********************************************************************
                                                       Multi-Edit Macro
                                                       26-12-1997 19:52

  Function: This function will return the indentation keyword.
  Entry   : -
  Exit    : indentation keyword

***********************************************************************/
{
  str C_Str;
  int iFoundLine;
  
  mark_pos;
  
  // ---------------------------------------------------------------------
  // Check whether we are in a EOL comment. If we are we must skip those
  // comments.
  // 
  eol;
  if ( GetStatusAtCursor( ) & _gsac_EOLComments ) {
    SkipNoiseFunc();
  }
  
  // ---------------------------------------------------------------------
  // Get first word, removing any extra space.
  // 
  First_Word;
  C_Str = Get_Word( gsDFEndOfWord );
  iFoundLine = C_Line;
  
  // ---------------------------------------------------------------------
  // If we are on a blank line, we should search the first filled line to
  // check the words there..
  // 
  if ( C_Str == "" ) {
    SkipNoiseFunc();
    First_Word;
    C_Str = Get_Word( gsDFEndOfWord );
    iFoundLine = C_Line;
  }

  // ---------------------------------------------------------------------
  // If we find the indicator killer, skip it..
  //
  if ( C_Str == "#$" ) {
    Word_Right;

    // Get the word after the indicator
    C_Str = Get_Word( gsDFEndOfWord );
    iFoundLine = C_Line;
  }
  
  // ---------------------------------------------------------------------
  // If the word starts with a bracket "[", we skip it, because we don't
  // need indicator, but the words after that.
  // 
  if ( Copy( C_Str, 1, 1 ) == "[" ) {
    // Skip the indicator
    First_Word;
    LangDoMatch( gsLANGUAGESHORT, "[" , "/NM=1/IC=0/LS=1" );
    Word_Right;

    // Get the word after the indicator
    C_Str = Get_Word( gsDFEndOfWord );
    iFoundLine = C_Line;
  }

  // ---------------------------------------------------------------------
  // If the first word is a special word then get the last word to
  // determine whether to indent or not. (With IF ... BEGIN, we don't
  // use IF, but we use BEGIN to determine to indent or not.)
  // 
  if ( WordFound( C_Str, gsSpecialIndentKeywords )) {
    eol;
    word_left;
    C_Str = Get_Word( gsDFEndOfWord );
  }

  // ---------------------------------------------------------------------
  // If the keyword was not found on the starting line and it's an
  // End-word, we ignore it, otherwise it shall be used with the auto-
  // unindent.
  // 
  if ( ( iFoundLine != C_Line ) & (WordFound( C_Str, gsUnindentKeywords ))) {
    C_Str = "";
  }

  goto_mark;

  return ( C_Str );
}  //  IndentKeyword


void AutoUnindent( str C_Str )
/********************************************************************
                                                       Multi-Edit Macro
                                                       26-12-1997 20:28

  Function: This function will automatically align the end-indenting
            keyword with the starting-indenting keyword.
  Entry   : the starting indenting keyword
  Exit    : -

***********************************************************************/
{
  str TStr[ max_Line_Length ];
  int Jx;
  
  C_Str = Caps( Remove_Space( C_Str ) );

  mark_pos;
  Up;
  First_Word;
  mark_pos;
  LangDoMatch( gsLANGUAGESHORT, C_Str, "/NM=1" );
  // Set indent level to first word on opening construct line
  First_Word;
  Set_Indent_Level;
  goto_mark;
  // Need to move line to start in indent column
  if ( Indent_Level > C_Col ) {
    TStr = Get_Word( "" );        // Get rest of the line
    Put_Line( "" );               // Blank the line
    Goto_Col( Indent_Level );
    Text( TStr );                 // Insert saved line at indent level
  }
  else {
    if ( indent_level < c_col ) {
      // Remove leading blanks
      Jx = C_Col - Indent_Level;
      Goto_Col( Indent_Level );
      Del_Chars( Jx );
    }
  }
  goto_mark;
  Goto_Col( Indent_Level );

}  //  AutoUnindent

void DFCmt( )
/******************************************************************************
															 Multi-Edit Macro
															 20-Oct-95  20:11

	Function: Auto adjust the end of line comments to the end of line column when
						enabled.  Called from the template macro.

	Entry   : /C=str				- The comment characters.
	Author	: Dan Hughes

							 Copyright (C) 1995 by American Cybernetics, Inc.
********************************************************************( ldh )***/
{
	str TStr = Parse_Str( "/C=", MParm_Str ) + " ";
  str sTmp;
	int SavRefresh 		= refresh;
	int SavInsertMode = Insert_Mode;
	int EndFunc				= false;
	int SavCol 				= 0;
  int bSkipWord;

	Push_Undo;
	Refresh = false;
	Insert_Mode = true;

  if ( gbEOLAdjustComment ) {
    if ( gbEOLAutoFillComment) {
      // -------------------------------------------------------------------
      // Check whether the EOL comment is the first word.
      // 
      Mark_Pos;
      First_Word;
      SavCol = C_Col;
      Goto_Mark;
      // -------------------------------------------------------------------
      // If it is not the first word then look for end_procedure
      // constructions and put the name of the procedure in the EOL comment.
      // 
      if ( C_Col != SavCol ) {
        Mark_Pos;
        First_Word;
        
        // ---------------------------------------------------------------
        // Skip the [] and #$ special signs...
        // 
        do {
          sTmp = Caps( Get_Word( gsDFEndOfWord ) );
          bSkipWord = (( sTmp == "[]" ) | ( sTmp == "#$"));
          if (bSkipWord) {
            Forward_Till    ( gsDFEndOfWord );
            Forward_Till_Not( gsDFEndOfWord );
          }
        } while ( bSkipWord );

        // ---------------------------------------------------------------
        // If it's an auto-fill-keyword, find the tagname.
        // 
        if ( WordFound( sTmp, gsEOLAutoFillKeywords )) {
          Left;                         // Make sure we are standing on the word
          Rm( "Language^Match" );       // Search the matching start-word (function etc.)
          Rm( "LangFindFunc" );         // Get the name to Return_Str
          Goto_Mark;

          if ( Length( Return_Str ) != 0 ) {
            TStr += Return_Str;
            EndFunc = true;
          }
        }
        else Goto_Mark;
      }
    }
    if ( C_Col < ( giEOLAdjustColumn - 1 ) && !EndFunc ) {
      Goto_Col( giEOLAdjustColumn );
		}
    else if (!EndFunc) TStr = " " + TStr;
	}
	Text( TStr );
  Goto_Col( C_Col );
	Insert_Mode = SavInsertMode;
	Refresh = SavRefresh;
	Pop_Undo;
}  // DFCmt