/******************************************************************************
 Copyright (c) 1997 G.J. Wijnands. All rights reserved.

														MULTI-EDIT MACRO FILE

 Name:   DataFlex

 Description:  This file contains macro's which make the access of the
               registry somewhat easier.

 EXTERNAL functions:

  str GetStringFromRegistry( str sSubkey, str sKey )
  void CreateSubKey( str sSubKey )

 INTERNAL functions:

  int GetRegString( int hRootReg, str sKey, str &sValue )
  int SetRegString( int hRootReg, str sKey, str sValue )

 $Header:: /MewAddOn/Multi Edit/dfreg.s 10    2-02-98 22:00 Gerard            $

******************************************************************************/
macro_file DFReg;

// MEW
#include WIN32.SH
#include DEBUG.SH
// Add-on
#include DFREG.SH
#include DATAFLEX.SH

import dword RegEnumKeyExA(dword KEY, DWORD dwIndex, pointer lpName,
  pointer lpcbName, Pointer lpReserved, pointer lpClass,
  pointer lpcbClass, pointer lpftLastWriteTime) advapi32 "RegEnumKeyExA";

// Registry access
structure _reg_temp32
{
  asciiz t1[ max_line_length ];
  int i1;
}


int DFRegOpenKeyEx( str sKey, int iOptions, int iSAM, int &hRoot )
/********************************************************************
                                                       Multi-Edit Macro
                                                         24-1-98 15:04

  Function: Opens a registry key which has as first specification the
            HKEY in which the real registry key is situated.
  Entry   : see RegOpenKeyEx. sKey is a combination of hKey and sSubkey
  Exit    : see RegOpenKeyEx

***********************************************************************/
{
  int iRetVal, hKey, iPos, hTmpRoot;
  str sHKey, sSubKey;

  // ---------------------------------------------------------------------
  // Get the rootkey
  //
  iPos = xPos( "\\", sKey, 1 );
  sHKey = Caps( Copy( sKey, 1, iPos - 1) );
  switch ( sHKey ) {
    case "HKEY_CLASSES_ROOT"     : hKey = HKEY_CLASSES_ROOT;     break;
    case "HKEY_CURRENT_USER"     : hKey = HKEY_CURRENT_USER;     break;
    case "HKEY_LOCAL_MACHINE"    : hKey = HKEY_LOCAL_MACHINE;    break;
    case "HKEY_USERS"            : hKey = HKEY_USERS;            break;
    case "HKEY_PERFORMANCE_DATA" : hKey = HKEY_PERFORMANCE_DATA; break;
    case "HKEY_CURRENT_CONFIG"   : hKey = HKEY_CURRENT_CONFIG;   break;
    case "HKEY_DYN_DATA"         : hKey = HKEY_DYN_DATA;         break;
    default                      : hKey = 0;                     break;
  }

  if ( hKey == 0) {
    iRetVal = giREGKEYNOTFOUND;
  }
  else {
    // ---------------------------------------------------------------------
    // Build the subkey (without the rootkey)
    //
    sSubkey = Copy( sKey, iPos + 1, Length( sKey ) - iPos );

    // ---------------------------------------------------------------------
    // Do the normal open
    //
    iRetVal = RegOpenKeyEx( hKey, sSubKey, iOptions, iSAM, hTmpRoot );

    // ---------------------------------------------------------------------
    // Pass the new handle to the calling function.
    //
    hRoot = hTmpRoot;
  }

  return ( iRetVal );

}  // DFRegOpenKeyEx

/******************************************************************************
															 Multi-Edit Macro
                               15-Apr-97  20:39

  Function: Gets a string from the registry.

  Entry   : hRootReg  - the handle to the root-registry
            sKey      - the keyword
            sValue    - -

  Exit    : sValue    - the value of the keyword
            returns   - the error which occured

               Copyright (C) 1997 by G.J. Wijnands
******************************************************************************/
int GetRegString( int hRootReg, str sKey, str &sValue )
{
  Struct _Reg_Temp32 RT;
  int TT;

  rt.i1 = max_line_length;
  iRegError=RegQueryValueEx( hRootReg, sKey,0, &TT, &rt.t1, &rt.i1  );
  if ( iRegError != giREGSUCCESS )
  {
    sValue = "";
  }
  else
  {
    sValue = Copy(rt.t1,1,rt.i1);
  }

  return ( iRegError );
}  //  GetRegString

/******************************************************************************
															 Multi-Edit Macro
                               15-Apr-97  21:20

  Function: To put a value into a key in the registry.

  Entry   : hRootReg  - the handle to the root-registry
            sKey      - the keyword
            sValue    - the (new) value of the keyword

  Exit    : None
            returns   - the error which occured

               Copyright (C) 1997 by G.J. Wijnands
******************************************************************************/
int SetRegString( int hRootReg, str sKey, str sValue)
{
  int tt;
  struct _reg_temp rt;

  rt.t1 = sValue;
  tt = REG_SZ;
  rt.i1 = length(rt.t1) + 1;
  iRegError=RegSetValueEx( hRootReg, sKey, 0,tt, &rt.t1, rt.i1 );

  return ( iRegError );

}  // SetRegString

/******************************************************************************
															 Multi-Edit Macro
                               24-Sep-97  21:55

  Function: Gets a string from the registry.

  Entry   : sSubkey   - the subkey which must be searched for the keyword
            sKey      - the key which must be searched

  Exit    : returns   - the requested value

               Copyright (C) 1997 by G.J. Wijnands
******************************************************************************/
str GetStringFromRegistry( str sSubkey, str sKey )
{
  int hRootReg;
  str sValue[200];

  if( (iRegError = DFRegOpenKeyEx( gsREGFLDROOT + sSubKey, 0, KEY_QUERY_VALUE , hRootReg )) == giREGSUCCESS)
    {
      GetRegString( hRootReg, sKey, sValue );
      RegCloseKey( hRootReg );
    }

  return ( sValue );
} // GetStringFromRegistry

int GetIntegerFromRegistry( str sSubkey, str sKey )
{
  int iRetVal;
  Val( iRetVal, GetStringFromRegistry( sSubKey,sKey ) );
  return ( iRetVal );
}  // GetIntegerFromRegistry

/******************************************************************************
															 Multi-Edit Macro
                               27-Sep-97  21:37

  Function: Sets a string of the registry.

  Entry   : sSubkey   - the subkey which must be searched for the keyword
            sKey      - the key which must be searched
            sValue    - the value the key must get

  Exit    : none

               Copyright (C) 1997 by G.J. Wijnands
******************************************************************************/
void SetKeyFromRegistry( str sSubkey, sKey, sValue )
{
  int hRootReg;

  if( (iRegError = DFRegOpenKeyEx( gsREGFLDROOT + sSubKey, 0, KEY_CREATE_SUB_KEY + KEY_SET_VALUE , hRootReg )) == giREGSUCCESS)
    {
      SetRegString( hRootReg, sKey, sValue );
      RegCloseKey( hRootReg );
    }

}  // SetKeyFromRegistry

/******************************************************************************
															 Multi-Edit Macro
                               27-Sep-97  21:37

  Function: Sets a string of the registry.

  Entry   : sSubkey   - the subkey which must be searched for the keyword
            sKey      - the key which must be searched
            sValue    - the value the key must get

  Exit    : none

               Copyright (C) 1997 by G.J. Wijnands
******************************************************************************/
void DeleteKeyFromRegistry( str sSubkey, str sKey )
{
  int hRootReg;

  if( (iRegError = DFRegOpenKeyEx( gsREGFLDROOT + sSubKey, 0, KEY_CREATE_SUB_KEY + KEY_SET_VALUE , hRootReg )) == giREGSUCCESS)
    {
      RegDeleteValue( hRootReg, sKey );
      RegCloseKey( hRootReg );
    }
}  // DeleteKeyFromRegistry

/******************************************************************************
															 Multi-Edit Macro
                               24-Sep-97  21:55

  Function: Gets a string from the part of the registry which is not used
						by the MEW addon.

  Entry   : sKey    - The total key of which the value must be retrieved.

  Exit    : returns - the requested value

               Copyright (C) 1997 by G.J. Wijnands
******************************************************************************/
str GetStringFromGlobalRegistry( str sKey )
{
  int hRootReg, iPos;
  str sValue[max_line_length], sFolder[ max_line_length ], sSubkey[max_line_length];

  // ---------------------------------------------------------------------
  // Parse key into folder and key
  //
  iPos = Length( sKey );
  While (( iPos > 0) && (Str_Char( sKey, iPos ) != "\\" )) iPos--;

  sFolder = Copy( sKey, 1, iPos - 1 );
  sSubkey = Copy( sKey, iPos + 1, Length( sKey ) - iPos );

  if( (iRegError = DFRegOpenKeyEx( sFolder, 0, KEY_QUERY_VALUE , hRootReg )) == giREGSUCCESS)
  {
    GetRegString( hRootReg, sSubKey, sValue );
    RegCloseKey( hRootReg );
  }

  return ( sValue );
}  // GetStringFromGlobalRegistry

/******************************************************************************
															 Multi-Edit Macro
                               24-Sep-97  21:56

  Function: To create a subkey

  Entry   : sSubkey - the subkey which must be created.

  Exit    : None

               Copyright (C) 1997 by G.J. Wijnands
******************************************************************************/
void CreateSubKey( str sSubKey )
{
  int hRootReg, hTmpRootReg, iDisposition;

  if( (iRegError = DFRegOpenKeyEx( gsREGFLDROOT , 0, KEY_CREATE_SUB_KEY + KEY_SET_VALUE , hRootReg )) == giREGSUCCESS)
  {
    iRegError = RegCreateKeyEx( hRootReg, sSubkey, 0,"",0,
      KEY_QUERY_VALUE | KEY_SET_VALUE | KEY_CREATE_SUB_KEY , 0, hTmpRootReg, iDisposition );

    RegCloseKey( hTmpRootReg );
    RegCloseKey( hRootReg );
  }

} // CreateSubKey

/******************************************************************************
															 Multi-Edit Macro
                               03-Jun-97  21:09

  Function: To get the default value of a key

  Entry   : sKey      - the keyword

  Exit    : returns   - the value of the keyword

               Copyright (C) 1997 by G.J. Wijnands
******************************************************************************/
str GetDefaultValue( str sKey )
{
  return (GetStringFromRegistry( gsREGFLDDEFAULT, sKey ));
}  // GetDefaultValue

/******************************************************************************
															 Multi-Edit Macro
                               30-Apr-97  21:20

  Function: To get the value of a key from a specific version of DataFlex

  Entry   : sLanguage - the version identifier
            sKey      - the keyword

  Exit    : returns   - the value of the keyword

               Copyright (C) 1997 by G.J. Wijnands
******************************************************************************/
str GetLanguageValue( str sLanguage, str sKey )
{
  return (GetStringFromRegistry( gsREGFLDLANGUAGES + sLanguage, sKey ));
}  // GetLanguageValue

/******************************************************************************
															 Multi-Edit Macro
                               30-Apr-97  21:20

  Function: To get the value of a key from a specific session.

  Entry   : sSession  - the sessionname
            sKey      - the keyword

  Exit    : returns   - the value of the keyword

               Copyright (C) 1997 by G.J. Wijnands
******************************************************************************/
str GetSessionValue( str sSession, str sKey )
{
  return (GetStringFromRegistry( gsREGFLDPROJECTS + sSession, sKey ));
}  // GetSessionValue

str GetSessionSettingRecursively( str sSession, str sKey )
{
  str sTmpSession = sSession;           // The session in which we recursively are.
  str sValue[200] = "";                 // The value of the key
  int bValueFound = FALSE;              // Has the value been found?
  int bInherit    = TRUE;               // Do we need to inherit again?

  // Search all projects recursively
  while ( !bValueFound && bInherit )
  {
    sValue = GetSessionValue( sTmpSession, sKey );
    bValueFound = iRegError == giREGSUCCESS;

    if ( !bValueFound )  // If key not found, then inherit.
    {
      // Determine which session to inherit from
      sTmpSession = GetSessionValue( sTmpSession, gsREGINHERIT );

      // Is the inherit key supplied?
      bInherit = iRegError == giREGSUCCESS;
    }
  }

  // if nothing has been found, try the language default.
  if ( !bValueFound )
  {
    sValue = GetLanguageValue(GetSessionValue( sSession , gsREGLANGUAGE ),sKey );
    bValueFound = iRegError == giREGSUCCESS;
  }

  // And if nothing was found there, try the global defaults
  if ( !bValueFound )
  {
    sValue = GetDefaultValue( sKey );
  }

  return ( sValue );
}  // GetSessionSettingRecursively

/******************************************************************************
															 Multi-Edit Macro
                               31-May-97  17:03

  Function: Gets the value of the specified key. Searches the "tree" from
            related projects (inherit). First the current session is searched.
            When nothing is found then via the keyword "Inherit" all other
            sessions are searched, until a value has been found.
            When the search at all related sessions has delivered nothing, the
            last place is searched: the language settings.

  Entry   : str - the key of which the value must be returned.

  Exit    : str - the first value of the key in the tree.

               Copyright (C) 1997 by G.J. Wijnands
******************************************************************************/
str GetStringSettingRecursively( str sKey )
{
  return (GetSessionSettingRecursively( gsProject, sKey ));
}  // GetStringSettingRecursively

int GetIntegerSettingRecursively( str sKey )
{
  int iRetVal;
  Val( iRetVal, GetSessionSettingRecursively( gsProject, sKey ) );
  return ( iRetVal );
}  // GetIntegerSettingRecursively

/********************************************************************
                                                       Multi-Edit Macro
                                                         22-1-98 21:29

  Function: This function will get all subkeys from the specified key
            and returns it as a string for the <dlg_choice> control.
  Entry   : sKey - the key from which the subkeys must be listed
            bSorted - (future) must the subkeys be sorted or not.
  Exit    : a string ready to be sent to a <dlg_choice> control.

***********************************************************************/
str GetSubkeys( str sKey, int bSorted, ... )
{
  int hRootReg, iRegError, iIndex;
  Struct _Reg_Temp RT;
  Struct _Reg_Temp Dummy;
  str sValue     [ max_line_length ];
  str sAllValues [ max_line_length ];
  str sSkipValues[ max_line_length ];

  // ---------------------------------------------------------------------
  // Get the optional values which should be skipped
  //
  if ( Get_Param_Type( 1 ) == 1 ) {
    sSkipValues = Get_Param_Str( 1 );
  }
  else sSkipValues = "";

  if( (iRegError = DFRegOpenKeyEx( sKey, 0, KEY_ENUMERATE_SUB_KEYS , hRootReg )) == giREGSUCCESS)
  {
    iIndex = 0;
    do {
      rt.i1 = 259;
      iRegError = RegEnumKeyExA( hRootReg, iIndex++, &rt.t1, &rt.i1, 0, 0, 0, &Dummy.t1 );
      if (iRegError == giREGSUCCESS)
      {
        sValue = Copy(rt.t1,1,rt.i1);
        if ( svl( sSkipValues ) ){
          if ( !XPOS( ","+Caps(sValue)+",", sSkipValues, 1 ) ) {
            sAllValues += sValue + "()";
          }
        }
        else sAllValues += sValue + "()";
      }
    } while ( iRegError == giREGSUCCESS );
    RegCloseKey( hRootReg );
  }
  return ( sAllValues );
}  // GetSubkeys
